// Copyright (c) 2012 The Foundry Visionmongers Ltd. All Rights Reserved.
#ifdef _WIN32
#include <FnPlatform/Windows.h>
#endif

#include <FnViewerModifier/plugin/FnViewerModifier.h>
#include <FnViewerModifier/plugin/FnViewerModifierInput.h>
#include <FnAttribute/FnGroupBuilder.h>
#include <FnAttribute/FnAttribute.h>
#ifdef __APPLE__
#include <OpenGL/OpenGL.h>
#include <OpenGL/glu.h>
#else
#include <GL/gl.h>
#include <GL/glu.h>
#endif

#include <iostream>



class CameraViewerModifier : public FnKat::ViewerModifier
{
public:

    CameraViewerModifier(FnKat::GroupAttribute args) : FnKat::ViewerModifier(args)
    {
        // Empty
    }


    static FnKat::ViewerModifier* create(FnKat::GroupAttribute args)
    {
        return (FnKat::ViewerModifier*)new CameraViewerModifier(args);
    }


    static FnKat::GroupAttribute getArgumentTemplate()
    {
        FnKat::GroupBuilder gb;
        return gb.build();
    }


    static const char* getLocationType()
    {
        return "camera";
    }


    void deepSetup(FnKat::ViewerModifierInput& input)
    {
        input.overrideHostGeometry();

    }


    void setup(FnKat::ViewerModifierInput& input)
    {
        // Empty
    }


    void draw(FnKat::ViewerModifierInput& input)
    {
        // Don't draw the camera representation if we're being looked through
        if (input.isLookedThrough())
        {
            return;
        }

        glPushAttrib(GL_POLYGON_BIT | GL_LIGHTING_BIT | GL_LINE_BIT);
        glDisable(GL_LIGHTING);
        // Lock global render context
//        UTIL::RenderContextLock localRenderLock(0);
        if (input.isSelected())
            glColor3f(1,1,1);
        else
            glColor3f(0,0,1);

	FnKat::FloatAttribute locatorScaleAttr = input.getGlobalAttribute( "viewer.default.drawOptions.locatorScale");
        if (locatorScaleAttr.isValid())
        {
          GLfloat locatorScale;
          locatorScale = locatorScaleAttr.getNearestSample(0)[0];
          glScalef(locatorScale, locatorScale, locatorScale);
        }

        glLineWidth(1);
        glPushMatrix();
        glBegin(GL_LINE_LOOP);
        glVertex3f(-0.5f,-0.5f,0.5f);
        glVertex3f(0.5f,-0.5f,0.5f);
        glVertex3f(0.5f,0.5f,0.5f);
        glVertex3f(-0.5f,0.5f,0.5f);
        glEnd();

        glBegin(GL_LINE_LOOP);
        glVertex3f(-0.5f,-0.5f,-0.5f);
        glVertex3f(0.5f,-0.5f,-0.5f);
        glVertex3f(0.5f,0.5f,-0.5f);
        glVertex3f(-0.5f,0.5f,-0.5f);
        glEnd();

        glBegin(GL_LINE_LOOP);
        glVertex3f(-0.5f,-0.5f,-0.75f);
        glVertex3f(0.5f,-0.5f,-0.75f);
        glVertex3f(0.5f,0.5f,-0.75f);
        glVertex3f(-0.5f,0.5f,-0.75f);
        glEnd();

        glBegin(GL_LINES);
        glVertex3f(0.5f, 0.5f, 0.5f);
        glVertex3f(0.5f, 0.5f, -0.5f);
        glVertex3f(0.5f, -0.5f, 0.5f);
        glVertex3f(0.5f, -0.5f, -0.5f);
        glVertex3f(-0.5f, 0.5f, 0.5f);
        glVertex3f(-0.5f, 0.5f, -0.5f);
        glVertex3f(-0.5f, -0.5f, 0.5f);
        glVertex3f(-0.5f, -0.5f, -0.5f);
        glEnd();

        glPopMatrix();
        glPopAttrib();
    }

    void cleanup(FnKat::ViewerModifierInput& input)
    {
        // Empty
    }

    void deepCleanup(FnKat::ViewerModifierInput& input)
    {
        // Empty
    };

    FnKat::DoubleAttribute getLocalSpaceBoundingBox(FnKat::ViewerModifierInput& input)
    {
        float locatorScale = 1.0;
        FnKat::FloatAttribute locatorScaleAttr = input.getGlobalAttribute( "viewer.default.drawOptions.locatorScale");
        if (locatorScaleAttr.isValid())
        {
          
          locatorScale = locatorScaleAttr.getNearestSample(0)[0];
        }        

	double bounds[6] = {
		locatorScale * -1, 
		locatorScale * 1, 
		locatorScale * -1, 
		locatorScale * 1, 
		locatorScale * -1, 
		locatorScale * 1};
        return FnKat::DoubleAttribute(bounds, 6, 1);
    }

    static void flush() {}

    static void onFrameEnd() {}

    static void onFrameBegin() {}
};

DEFINE_VMP_PLUGIN(CameraViewerModifier)

void registerPlugins()
{
    REGISTER_PLUGIN(CameraViewerModifier, "ZCameraViewerModifier", 0, 1);
}
